//
//  Biblioteca.m
//  PowerComicReader
//
//  Created by Sebastian Suelves Selles on 29/03/13.
//  Copyright (c) 2013 Sebastian Suelves Selles. All rights reserved.
//

#import "Biblioteca.h"
#import "UnRAR.h"
#import "MiniZip.h"


@interface Biblioteca ()
@property (strong,nonatomic) NSMutableArray* comics;
@property (nonatomic, strong) CDataColeccion *coleccion;
@property (nonatomic, strong) NSFetchedResultsController *cdComicsColeccion;

@end

@implementation Biblioteca


-(id) init
{
    return [self initWithColeccion:nil ofColeccion:nil];
}

//inicializador designado
- (id)initWithColeccion:(NSFetchedResultsController *)comicsColeccion ofColeccion:(CDataColeccion *)coleccion
{
    self = [super init];
    
    if (self)
    {
        //inicializamos la coleccion
        self.coleccion = coleccion;
        
        self.comics=[[NSMutableArray alloc]init];
        self.cdComicsColeccion = comicsColeccion;

        NSArray *array = [comicsColeccion fetchedObjects];
            
        for (NSInteger indice=0; indice<[array count]; indice++)
        {
            //cargamos los datos desde core data y los cargamos en nuestro modelo
            CDataColeccion *coleccionTemp = (CDataColeccion *) [array objectAtIndex:indice];
            
            NSSortDescriptor *sortDescriptor;
            sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"ordenColeccion" ascending:YES];
            NSArray *sortDescriptors = [NSArray arrayWithObject:sortDescriptor];
            
            
            NSArray *arrayComics = [[coleccionTemp.coleccionComic allObjects] sortedArrayUsingDescriptors:sortDescriptors];
            self.coleccionActiva = coleccion.coleccion;
            
            for (CDataComic *cdComic in arrayComics) {
                NSLog(@"%@", cdComic);
                Comic *comic = [[Comic alloc] initWithColeccion:coleccionTemp.coleccion AndOrdenColeccion:cdComic.ordenColeccion AndPaginaActual:cdComic.paginaActual
                                                  AndNumPaginas:cdComic.numeroPaginas AndDirectorio:cdComic.nombreCarpeta AndExtensionImagenes:cdComic.extensionImagenes];
                NSLog(@"%@ , %@", comic.directorio, comic.ordenColeccion);
                //añadimos el comic a nuestro modelo
                [self insertComicsObject:comic inComicsAtIndex:[self.comics count] AndInsertInCoreData:NO];
            }
        }
        
        //importamos los comics y los reordenamos
        [self importaComics];
        [self reordenaComics];
    }
    return self;
}

//getter
-(NSMutableArray *) comics
{
    if (!_comics) {
        _comics=[[NSMutableArray alloc]init];
    }
    return _comics;
}


-(void) trasladaComic:(Comic *)comic toColeccion:(NSString *)coleccionDestino
{
    //obtenemos el comic y la coleccion destino
    CDataComic *cdComic = [self findComicCoreData:comic];
    CDataColeccion *cdColeccion = [self findColeccionCoreData:coleccionDestino];
    
    //actualizamos en el comic la coleccion destino
    cdComic.comicColeccion = cdColeccion;
    
    //grabamos los cambios
    [self commitCoreData:self.managedObjectContext];
}

-(void) removeComicsAtIndex:(NSUInteger) indice
{
    //borramoe el comic del modelo, de core data, y los ficheros fisicos
    if (indice < [self.comics count]) {
        Comic *comic = [self.comics objectAtIndex:indice];
        [self.comics removeObjectAtIndex:indice];
        [self removeComicCoreData:comic];
        [self borraFicherosComic:comic];
        [self reordenaComics];
    }
    else
    {
        NSLog(@"no removido: indice=%lu y count: %lu", (unsigned long)indice, (unsigned long)[self.comics count]);
    }
}

-(void) moveComicFromIndex:(NSUInteger) desde toIndex:(NSUInteger) hasta
{
    //cambiamos la posicion del comic dentro de la coleccion
    Comic* comicHasta = [self.comics objectAtIndex:hasta];
    [self.comics replaceObjectAtIndex:hasta withObject:[self.comics objectAtIndex:desde]];
    [self.comics replaceObjectAtIndex:desde withObject:comicHasta];
    [self reordenaComics];
}

-(Comic *) getComicAtIndex:(NSUInteger) indice
{
    //devuelve el comic que se encuentra en la posicion indicada por el indice
    return self.comics[indice];
}

-(NSMutableArray *) getAllComics
{
    //devuelve una copia de los comics de la coleccion
    return [self.comics mutableCopy];
}

-(NSUInteger) getComicsCount
{
    //devuelve el numero de comics de la coleccion
    return [self.comics count];
}

-(NSMutableArray *) getComicsColleccion:(NSString*) coleccion
{
    //Innecesario
}

-(void) removeComicsObject:(Comic *)object
{
    //borra un comic y sus datos relacionados
    [self.comics removeObject:object];
    [self removeComicCoreData:object];
    [self borraFicherosComic:object];
    [self reordenaComics];
}

-(void) insertComicsObject:(Comic *)object inComicsAtIndex:(NSUInteger)index AndInsertInCoreData:(BOOL) insertarCoreData
{
    //inserta un comic
    [self.comics insertObject:object atIndex:index];
    if (insertarCoreData) {
        [self insertaComicCoreData:object];
    }
    [self reordenaComics];
}
-(NSMutableArray *) getColecciones
{
    //Innecesario
}

-(void) importaComics
{
    NSString *documentsDir;
    NSString *extension;
 
    //obtenemos el directorio Documents de la aplicacion
    NSArray* paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    documentsDir = [paths objectAtIndex:0];
    NSLog(@"Document dir: %@", documentsDir);
    
    //obtenemos los ficheros que existen en un directorio
    NSArray* ficherosDirectorio = [[NSFileManager defaultManager]
                                                    contentsOfDirectoryAtPath:documentsDir error:nil];
    
    for (NSString* fich in ficherosDirectorio)
    {
        //obtenemos la extension de los ficheros, si es un formato conocido lo procesamos
        extension = [fich pathExtension];
        if (![extension caseInsensitiveCompare:@"cbr"] || ![extension caseInsensitiveCompare:@"cbz"] ||
            ![extension caseInsensitiveCompare:@"rar"] || ![extension caseInsensitiveCompare:@"zip"])
        {
            //Descomprimimos el fichero
            [self procesaFicheroComic:fich];
            
            //Borramos el fichero procesado
            NSString *fichBorrar = [documentsDir stringByAppendingPathComponent:fich];
            [[NSFileManager defaultManager] removeItemAtPath:fichBorrar error:NULL];
        }
    }
}

-(void) eliminaBiblioteca
{
    //Innecesario
}

-(void) refrescaBiblioteca
{
    //reimportamos comics y reordenamos
    [self importaComics];
    [self reordenaComics];
}

-(void) saveMiniatura:(NSString*) directorioComic desdeImagen:(UIImage *)portada
{
    //generamos la miniatura a partir de la portada del comic
    
    UIGraphicsBeginImageContext(CGSizeMake(202.0f, 288.0f));
    
    [portada drawInRect:CGRectMake(0, 0, 202.0f, 288.0f)];
    
    UIImage * miniatura = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    
    //grabamos la miniatura como 0.png
    NSString *ficheroMiniatura = [directorioComic stringByAppendingPathComponent:@"0.png"];
    [UIImagePNGRepresentation(miniatura) writeToFile:ficheroMiniatura atomically:YES];
}

-(void) procesaFicheroComic:(NSString *)fichero
{
    id archive = nil;
    NSString *documentsDir;
    NSFileManager* fm;
    NSString *ficheroComprimido;
    NSNumber* numeroPaginasComic;
    NSString* carpetaComic;
    NSString* dirDescompresion;
    NSString* extensionComic;
    
    //obtenemos el directorio Documents de la aplicacion
    NSArray* paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    documentsDir = [paths objectAtIndex:0];
    NSLog(@"Document dir: %@", documentsDir);
    
    ficheroComprimido = [NSString stringWithFormat:@"%@/%@", documentsDir, fichero];
    
    //Creamos dentro de documents la carpeta donde descomprimir el comic
    carpetaComic = [self creaCarpetaDescompresion:ficheroComprimido];
    NSLog(@"Fichero Comprimido: %@", ficheroComprimido);
    NSLog(@"Carpeta comic: %@", carpetaComic);
    if (![carpetaComic isEqualToString:@""])
    {
        //calculamos el directorio donde descomprimir el comic
        dirDescompresion = [documentsDir stringByAppendingPathComponent:@"Imported Comics"];
        dirDescompresion = [dirDescompresion stringByAppendingPathComponent:carpetaComic];
        
        //Descomprimimos el comic y obtenemos el numero de paginas que posee
        numeroPaginasComic = [self descomprimirFichero:ficheroComprimido enDirectorioDescompresion:dirDescompresion yDetectarExtension:&extensionComic];
        
        NSLog(@"NumPaginasComic: %@, extension: %@", numeroPaginasComic, extensionComic);
        
        //calculamos su orden dentro de la coleccion
        NSNumber *ordenColeccion = [NSNumber numberWithUnsignedInt:[self getComicsCount]+1];
        
        //creamos el comic en memoria
        Comic *comic = [[Comic alloc]initWithColeccion:@"Comics" AndOrdenColeccion:ordenColeccion AndPaginaActual:@1 AndNumPaginas:numeroPaginasComic AndDirectorio:carpetaComic AndExtensionImagenes:extensionComic];
        
        //insertamos el comic en el modelo y core data
        [self insertComicsObject:comic inComicsAtIndex:0 AndInsertInCoreData:YES];
    }
}

- (void) insertaComicCoreData:(Comic *)comic
{
    //insertamos el comic en core data
    NSEntityDescription *comicEntityDescription = (NSEntityDescription *)[NSEntityDescription entityForName:@"CDataComic" inManagedObjectContext:self.coleccion.managedObjectContext];
    CDataComic *cdComic = (CDataComic *) [[NSManagedObject alloc] initWithEntity:comicEntityDescription insertIntoManagedObjectContext:self.coleccion.managedObjectContext];
    
    //hacemos un wrapper entre las clases del modelo y las de core data
    cdComic.ordenColeccion = comic.ordenColeccion;
    cdComic.numeroPaginas = comic.numeroPaginas;
    cdComic.nombreCarpeta = comic.directorio;
    cdComic.paginaActual = comic.paginaActual;
    cdComic.extensionImagenes = comic.extensionImagenes;
    
    //añadimos el objeto y hacemos commit
    [self.coleccion addColeccionComicObject:cdComic];
    
    [self commitCoreData:self.coleccion.managedObjectContext];
}

-(void) removeComicCoreData:(Comic *)comic
{
    //borramos el comic de core data y hacemos commit
    CDataComic *cdComic = [self findComicCoreData:comic];
    
    [self.managedObjectContext deleteObject:cdComic];
    
    [self commitCoreData:self.managedObjectContext];
}

-(void) updateComicCoreData:(Comic *)comic
{
    //actualizamos el comic y hacemos commit
    CDataComic *cdComic = [self findComicCoreData:comic];
    cdComic.ordenColeccion = comic.ordenColeccion;
    cdComic.paginaActual = comic.paginaActual;
    cdComic.numeroPaginas = comic.numeroPaginas;
    cdComic.nombreCarpeta = comic.directorio;
    cdComic.extensionImagenes = comic.extensionImagenes;
    [self commitCoreData:self.managedObjectContext];
}

-(void) reordenaComics
{
    //actualizamos el orden en el array de comics, actualizamos los datos en core data y hacemos commit
    [self updateOrdenArrayComics];
    [self updateOrdenCoreData];
    [self commitCoreData:self.managedObjectContext];
}

- (void)updateOrdenArrayComics
{
    for (NSInteger indice=0;indice < [self.comics count]; indice++)
    {
        //actualizamos el orden de los comics de la coleccion en el modelo
        Comic *comic = [self.comics objectAtIndex:indice];
        comic.ordenColeccion = [NSNumber numberWithUnsignedInteger:indice];
    }
}

- (void)updateOrdenCoreData
{
    for (Comic *comic in self.comics) {
        //actualizamos orden en core data
        [self updateComicCoreData:comic];
    }
    //grabamos los cambios
    [self commitCoreData:self.managedObjectContext];
}

-(void) commitCoreData:(NSManagedObjectContext *) moc
{
    //implementacion del commit core data
    NSError *error;
    if (![moc save:&error])
    {
        NSLog(@"Error en Commit");
    }
}

-(CDataComic *) findComicCoreData:(Comic *)comic
{
    //buscamos el comic en core data
    NSFetchRequest * fetchRequest = [[NSFetchRequest alloc] init];
    
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"CDataComic" inManagedObjectContext:self.managedObjectContext];
    [fetchRequest setEntity:entity];
    
    //localizamos el comic por el nombre de la carpeta que es el del comic mediante un predicado (where)
    NSPredicate *pred = [NSPredicate predicateWithFormat:@"nombreCarpeta = %@", comic.directorio];
    [fetchRequest setPredicate:pred];
    
    //ejecutamos la consulta
    NSError *error;
    NSArray *foundObjects = [self.managedObjectContext executeFetchRequest:fetchRequest error:&error];
    if (foundObjects == nil)
    {
        FATAL_CORE_DATA_ERROR(error);
        return;
    }
    
    //devolvemos el objeto leido
    return [foundObjects objectAtIndex:0];
}

-(CDataColeccion *) findColeccionCoreData:(NSString *)coleccion
{
    //buscamos la coleccion en core data
    NSFetchRequest * fetchRequest = [[NSFetchRequest alloc] init];
    
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"CDataColeccion" inManagedObjectContext:self.managedObjectContext];
    [fetchRequest setEntity:entity];
    
    //filtramos por coleccion
    NSPredicate *pred = [NSPredicate predicateWithFormat:@"coleccion = %@", coleccion];
    [fetchRequest setPredicate:pred];
    
    //ejecutamos la consulta
    NSError *error;
    NSArray *foundObjects = [self.managedObjectContext executeFetchRequest:fetchRequest error:&error];
    if (foundObjects == nil)
    {
        FATAL_CORE_DATA_ERROR(error);
        return;
    }
    
    //devolvemos el objeto leido
    return [foundObjects objectAtIndex:0];
}

- (NSString*) creaCarpetaDescompresion:(NSString *)ficheroComprimido
{
    id archive = nil;
    NSString *documentsDir;
    NSString* directorioComic;
    NSFileManager* fm;
    NSString *retornoCarpetaDescompresion;
    
    //obtenemos el directorio Documents de la aplicacion
    NSArray* paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    documentsDir = [paths objectAtIndex:0];
    NSLog(@"Document dir: %@", documentsDir);
    
    NSString *extension = [ficheroComprimido pathExtension];
    
    //en funcion del tipo de fichero usamos un descompresor u otro
    if (![extension caseInsensitiveCompare:@"cbr"] || ![extension caseInsensitiveCompare:@"rar"])
    {
        archive = [[UnRAR alloc] initWithArchiveAtPath:ficheroComprimido];
    }
    else
    {
        archive = [[MiniZip alloc] initWithArchiveAtPath:ficheroComprimido];
    }
    
    [archive setSkipInvisibleFiles:YES];
    NSString* cover = nil;
    
    //obtenemos la lista de ficheros dentro del fichero comprimido
    NSArray *ficheros = [archive retrieveFileList];
    
    NSString* directorio = [ficheros[0] stringByDeletingLastPathComponent];
    
    //si el fichero comprimido no contiene un directorio, asignamos como nombre de fichero comprimido
    if  ([directorio isEqualToString:@""])
    {
        directorio = [ficheroComprimido stringByDeletingPathExtension];
    }
    NSLog(@"fichero[0]: %@", ficheros[0]);
    NSLog(@"Directorio base: %@", directorio);
    
    //Crear Directorio
    fm = [NSFileManager defaultManager];
    
    BOOL esDirectorio;
    
    //creamos el directorio del comic, si ya existia omitimos la importacion
    directorioComic = [[documentsDir stringByAppendingPathComponent:@"Imported Comics"] stringByAppendingPathComponent:directorio];
    if ([fm fileExistsAtPath:directorioComic isDirectory:&esDirectorio] && esDirectorio)
    {
        NSLog(@"El directorio ya existe omitimos la importacion");
        return @"";
    }
    else
    {
        NSLog(@"No existe directorio, lo creamos e importamos");
        
        if ([fm createDirectoryAtPath:directorioComic withIntermediateDirectories:YES attributes:nil error:NULL])
        {
            NSLog(@"Directorio creado");
            return directorio;
        }
    }
}

- (NSNumber *) descomprimirFichero:(NSString *)ficheroComprimido enDirectorioDescompresion:(NSString*)directorioDescompresion yDetectarExtension:(NSString **)extensionComic
{
    NSNumber* contadorPaginas;
    NSString* extension;
    
    contadorPaginas = @0;
    
    id archive = nil;
    
    extension = [ficheroComprimido pathExtension];
    
    //en funcion de la extension del fichero usamos un descompresor u otro
    if (![extension caseInsensitiveCompare:@"cbr"] || ![extension caseInsensitiveCompare:@"rar"])
    {
        archive = [[UnRAR alloc] initWithArchiveAtPath:ficheroComprimido];
    }
    else
    {
        archive = [[MiniZip alloc] initWithArchiveAtPath:ficheroComprimido];
    }
    [archive setSkipInvisibleFiles:YES];
    
    //obtenemos la lista de los ficheros dentro del fichero comprimido, ordenando por nombre
    NSMutableArray* listaFicheros = [NSArray arrayWithArray:[[archive retrieveFileList] sortedArrayUsingComparator:^(NSString* a, NSString* b) {
        return [a compare:b options:NSNumericSearch];
    }]];
    
    for (NSString* file in listaFicheros)
    {
        //descomprimimos los ficheros de imagenes
        extension = [file pathExtension];
        if (![extension caseInsensitiveCompare:@"jpg"] || ![extension caseInsensitiveCompare:@"jpeg"] ||
            ![extension caseInsensitiveCompare:@"png"] || ![extension caseInsensitiveCompare:@"gif"]) {
            NSLog(@"fich: %@", file);
            
            //contamos las paginas descomprimidas
            unsigned int contador = [contadorPaginas unsignedIntValue] + 1;
            
            contadorPaginas = [NSNumber numberWithUnsignedInt:contador];
            
            //calculamos el destino del fichero descomprimido renombrando cada uno de los ficheros a partir del 1.extension
            NSString* destino = [[directorioDescompresion stringByAppendingPathComponent:[contadorPaginas stringValue]] stringByAppendingFormat:@".%@",extension];
            NSLog(@"Destino: %@", destino);
            
            //descomprimimos el fichero
            if ([archive extractFile:file toPath:destino]) {
                NSLog(@"Fichero descomprimido cbr: %@", destino);
            }
        }
    }
    
    *extensionComic = [NSString stringWithFormat:@".%@",extension];
    
    //generamos la miniatura a partir de la portada
    [self saveMiniatura:directorioDescompresion desdeImagen:[UIImage imageWithContentsOfFile:[[directorioDescompresion stringByAppendingPathComponent:@"1"] stringByAppendingFormat:@".%@",extension]]];
    
    //devolvemos el numero de paginas descomprimidas
    return contadorPaginas;
}

-(void) borraFicherosComic:(Comic *)comic
{
    //borramos los ficheros que forman un comic borrando simplemente el directorio que lo contiene que automaticamente borra todo su contenido
    NSArray* paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDir = [paths objectAtIndex:0];
    
    NSString *fichBorrar = [documentsDir stringByAppendingPathComponent:@"Imported Comics"];
    fichBorrar = [fichBorrar stringByAppendingPathComponent:comic.directorio];
    NSLog(@"Directorio a borrar: %@", fichBorrar);
    [[NSFileManager defaultManager] removeItemAtPath:fichBorrar error:NULL];
    
}

- (NSManagedObjectContext *)managedObjectContext
{
    //obtenenmos el manejador de contexto de core data
    NSManagedObjectContext *context = nil;
    id delegate = [[UIApplication sharedApplication] delegate];
    if ([delegate performSelector:@selector(managedObjectContext)]) {
        context = [delegate managedObjectContext];
    }
    return context;
}

@end
