//
//  EstanteriaViewController.m
//  PowerComicReader
//
//  Created by Sebastian Suelves Selles on 17/03/13.
//  Copyright (c) 2013 Sebastian Suelves Selles. All rights reserved.
//

#import "BibliotecaViewController.h"
#import "Comic.h"
#import "CelulaComic.h"
#import "Biblioteca.h"
#import "VisualizadorComicViewController.h"
#import "LXReorderableCollectionViewFlowLayout.h"
#import "PreferenciasUsuario.h"
#import "ModalAlertDelegate.h"




@interface BibliotecaViewController () <UICollectionViewDataSource, LXReorderableCollectionViewDelegateFlowLayout, ColeccionViewDelegate, UIPopoverControllerDelegate>

//IBounds
@property (weak, nonatomic) IBOutlet UIToolbar *toolbar;
@property (weak, nonatomic) IBOutlet UIBarButtonItem *botonColecciones;
@property (weak, nonatomic) IBOutlet UIBarButtonItem *botonEditar;
@property (weak, nonatomic) IBOutlet UIBarButtonItem *botonTrasladar;
@property (weak, nonatomic) IBOutlet UIBarButtonItem *botonEliminar;
@property (weak, nonatomic) IBOutlet UIBarButtonItem *botonSeleccionarTodo;
@property (weak, nonatomic) IBOutlet UIBarButtonItem *botonRefrescar;
@property (weak, nonatomic) IBOutlet UICollectionView *collectionView;
@property (weak, nonatomic) IBOutlet UIBarButtonItem *botolLabelColeccion;

//Array para llevar la cuenta de que comics estan seleccionados en modo edicion
@property (strong, nonatomic) NSMutableArray *comicsSeleccionados;

//Copia de elementos del toolbar para permitir ocultar y desocultar botones
@property (strong, nonatomic) NSMutableArray     *todosElementosToolbar;


//Booleanos para los distintos estados
@property (nonatomic) BOOL modoEdicion;
@property (nonatomic)BOOL estaSeleccionadoTodo;
@property (nonatomic) BOOL trasladandoComic;

//objeto biblioteca
@property (strong, nonatomic) Biblioteca *biblioteca;
//datos leidos coredata
@property (strong, nonatomic) NSFetchedResultsController *fetchedResultsController;
//coleccion activa
@property (strong, nonatomic) NSString *coleccionActiva;


@end

@implementation BibliotecaViewController

-(void) actualizaBotonLabelColeccion:(NSString *)texto;
{
    //Actualiza el texto del boton Coleccion, no podemos usar un label en un toolbar, por lo que usamos un boton sin borde para emular la etiqueta, ya que el acceso a los botones del toolbar no es directo debe buscarse dentro de los items que contiene el toolbar, para ello asignamos un TAG=4 a dicho boton de modo que podamos distingirlo con facilidad
    
    self.botolLabelColeccion.title = texto;
    
    NSMutableArray *array = [self.toolbar.items mutableCopy];
    for (NSInteger indice=0;indice<[array count]; indice++)
    {
        UIBarButtonItem *item = [array objectAtIndex:indice];
        if (item.tag == 4)
        {
            item.title = texto;
            [array replaceObjectAtIndex:indice withObject:item];
            break;
        }
    }
    self.toolbar.items = array;
    
    //Hacemos el trabajo doble pues tenemos que modificar tanto el toolbar visible como el de reserva que alberga todos los controles y nos permite eliminar elementos sobre la copia
    NSMutableArray *arrayTodos = [self.todosElementosToolbar mutableCopy];
    for (NSInteger indice=0;indice<[arrayTodos count]; indice++)
    {
        UIBarButtonItem *item = [arrayTodos objectAtIndex:indice];
        if (item.tag == 4)
        {
            item.title = texto;
            [arrayTodos replaceObjectAtIndex:indice withObject:item];
            break;
        }
    }
    //una vez modificado el boton guardamos la ultima copia
    self.todosElementosToolbar = arrayTodos;
}


-(Biblioteca *)biblioteca //getter del objeto biblioteca
{
    if (!_biblioteca)
    {
        //Comprobamos si existe la coleccion "Comics"
        [self compruebaExisteColeccion];
        //Cargamos desde Core Data los datos de la ultima coleccion usada desde las preferencias almacenadas
        [self cargaDatosColeccion];
        //Cargamos los datos de los comics que componen la coleccion
        [self fetchBiblioteca];
        //Generamos la biblioteca a partir de los datos obtenidos
        _biblioteca = [[Biblioteca alloc] initWithColeccion:self.fetchedResultsController ofColeccion:self.coleccion];

        //actualizamos la etiqueta que indica cual es la coleccion activa
        [self actualizaBotonLabelColeccion:_biblioteca.coleccionActiva];
    }
    return _biblioteca;
}


-(void)actualizaBiblioteca //actualizamos los datos que se visualizan
{
    
    
    //recargamos los datos de la coleccion y los comics de la biblioteca
    [self cargaDatosColeccion];
    [self fetchBiblioteca];
    self.biblioteca = [self.biblioteca initWithColeccion:self.fetchedResultsController ofColeccion:self.coleccion];

    //actualizamos la etiqueta y recargamos el collectionview
    [self actualizaBotonLabelColeccion:self.biblioteca.coleccionActiva];
    [self.collectionView reloadData];
}

-(void)cargaDatosColeccion
{
    //Preparamos para leer datos de Core Data
    NSFetchRequest * fetchRequest = [[NSFetchRequest alloc] init];
    
    //Indicamos que entidad vamos a leer
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"CDataColeccion" inManagedObjectContext:self.managedObjectContext];
    [fetchRequest setEntity:entity];
    
    //Definimos las condiciones de seleccion de los datos (Where en SQL)
    NSPredicate *pred = [NSPredicate predicateWithFormat:@"coleccion = %@", self.coleccionActiva];
    //aplicamos el predicado a los datos
    [fetchRequest setPredicate:pred];
    
    //ejecutamos la consulta
    NSError *error;
    NSArray *foundObjects = [self.managedObjectContext executeFetchRequest:fetchRequest error:&error];
    if (foundObjects == nil)
    {
        FATAL_CORE_DATA_ERROR(error);
        return;
    }
    
    //Obtenemos los datos de la coleccion
    NSArray *array = [self.managedObjectContext executeFetchRequest:fetchRequest error:&error];
    if ([array count]>0)
    {
        self.coleccion = [array objectAtIndex:0];
    }
}

-(NSMutableArray *)comicsSeleccionados //getter comicsSeleccionados
{
    if (!_comicsSeleccionados)
    {
        //inicializamos la primera vez que accedemos al objeto dicho objeto
        _comicsSeleccionados = [[NSMutableArray alloc] init];
    }
    return _comicsSeleccionados;
}

- (IBAction)botonRefrescarPulsado:(UIBarButtonItem *)sender
{
    //refrescamos las preferencias
    [self personalizaEstilo];
    
    //Intentamos reimportar nuevos comics al pulsar el boton de refrecado
    [self.biblioteca refrescaBiblioteca];
    //actualizamos el collectionview con los nuevos datos
    [self actualizaBiblioteca];
}

- (IBAction)botonEliminarPulsado:(UIBarButtonItem *)sender
{
    NSLog(@"Pulsado Eliminar");
    
    //AlertView modal para pedir confirmacion de borrado
    NSString *mensajeAlertView = NSLocalizedString(@"deleteConfirmation", @"This action will delete %d comics\nAre you sure?");
    mensajeAlertView = [NSString stringWithFormat:mensajeAlertView, [self.comicsSeleccionados count]];
    NSString *tituloAlertView = NSLocalizedString(@"Attention", @"Attention");
    UIAlertView* confirmaBorrado = [[UIAlertView alloc] initWithTitle:tituloAlertView message:mensajeAlertView delegate:nil
                                                    cancelButtonTitle:NSLocalizedString(@"No", @"No")
                                                    otherButtonTitles:NSLocalizedString(@"Yes", @"Yes"), nil];
    confirmaBorrado.alertViewStyle = UIAlertViewStyleDefault;
    ModalAlertDelegate *delegate = [ModalAlertDelegate delegateWithAlert:confirmaBorrado];
    NSString* respuesta;
    NSUInteger botonPulsado;
    botonPulsado = [delegate show];
    
    //si pulsamos "Si" borramos los comics seleccionados
    if (botonPulsado==1)
    {
        for (Comic *comic in [self.comicsSeleccionados copy])
        {
            //borramos los ficheros que conforman el comics
            [self.biblioteca removeComicsObject:comic];
            //borramos el comic del modelo de datos
            [self.comicsSeleccionados removeObject:comic];
        }
        dispatch_async(dispatch_get_main_queue(), ^{
            //recargar datos collectionview
            [self.collectionView reloadData];
        });
    }
}

- (IBAction)botonSeleccionarTodoPulsado:(UIBarButtonItem *)sender {
    NSLog(@"Pulsado Seleccionar Todo");
    //cambiamos el texto del boton en funcion de si hemos seleccionado todo o no previamente
    if (!self.estaSeleccionadoTodo)
    {
        //seleccionar todo
        //deseleccionamos todo los objetos que llevamos en seguimiento para edicion
        [self.comicsSeleccionados removeAllObjects];
        
        //hacemos la animacion de la seleccion (marco azul)
        for (NSIndexPath *index in [self.collectionView indexPathsForVisibleItems]) {
            [self.collectionView selectItemAtIndexPath:index animated:YES scrollPosition:UICollectionUpdateActionNone];
        }
        
        //metemos todos los comics de la biblioteca en seguimiento
        self.comicsSeleccionados = [self.biblioteca getAllComics];
        self.botonEliminar.enabled = YES;
        self.botonTrasladar.enabled = YES;
        
        NSString *deselecciona = NSLocalizedString(@"Select None", @"BibliotecaViewController Select None");
        
        //cambiamos el texto del boton
        [self.botonSeleccionarTodo setTitle:deselecciona];
        self.estaSeleccionadoTodo = YES;
    }
    else
    {
        //deseleccionamos todo
        
        //limpiamos la lista de comics en seguimiento
        [self.comicsSeleccionados removeAllObjects];
        
        //Deseleccionamos todos los comics del collectionView
        for (NSIndexPath *index in [self.collectionView indexPathsForVisibleItems]) {
            [self.collectionView deselectItemAtIndexPath:index animated:YES];
        }
        self.botonEliminar.enabled = NO;
        self.botonTrasladar.enabled = NO;
        
        //cambiamos el titulo del boton
        NSString *selecciona = NSLocalizedString(@"Select All", @"BibliotecaViewController Select All");
        [self.botonSeleccionarTodo setTitle:selecciona];
        self.estaSeleccionadoTodo = NO;
    }
    
}


- (IBAction)botonEditarPulsado:(UIBarButtonItem *)sender {
    //TODO
     NSLog(@"Pulsado Editar");
    
    self.modoEdicion = !self.modoEdicion;
    
    //control del boton de editar
    if (self.modoEdicion)
    {
        NSMutableArray *items = [self.todosElementosToolbar mutableCopy];
        
        //ocultamos el botno de colecciones en modo edicion
        [items removeObject:self.botonColecciones];
        self.toolbar.items = items;
        
        self.botonEditar.style = UIBarButtonItemStyleDone;
        self.botonEditar.title = @"Ok";
        
        //permitimos seleccion multiple
        [self.collectionView setAllowsMultipleSelection:YES];
    }
    else
    {
        //recuperamos los botones del toolbar y los adecuamos correctamente
        NSMutableArray *items = [self.todosElementosToolbar mutableCopy];
        
        [items removeObject:self.botonTrasladar];
        [items removeObject:self.botonEliminar];
        [items removeObject:self.botonSeleccionarTodo];
        self.toolbar.items = items;
        
        self.botonEditar.style = UIBarButtonItemStyleBordered;
        
        //cambiamos el titulo del boton
        NSString *editar = NSLocalizedString(@"ColeccionEdit", @"BibliotecaViewController Edit");
        self.botonEditar.title = editar;
        
        //desactivamos la seleccion multiple
        [self.collectionView setAllowsMultipleSelection:NO];
        
        //deseleccionamos los objetos seleccionados anteriormente
        for (NSIndexPath *indice in self.collectionView.indexPathsForSelectedItems) {
            NSLog(@"objeto seleccionado");
            
            [self.collectionView deselectItemAtIndexPath:indice animated:NO];
        }
        //limpiamos la lista de comics a tratar
        NSLog(@"objetos seleccionados: %lu", (unsigned long)[self.comicsSeleccionados count]);
        [self.comicsSeleccionados removeAllObjects];
    }
}

- (void)viewDidLoad
{
    [super viewDidLoad];
	// Do any additional setup after loading the view.
    //ajustamos el background en funcion de los datos de preferencia
    
    [self personalizaEstilo];
    
    self.modoEdicion = NO;
    
    self.botonEliminar.enabled = NO;
    self.botonTrasladar.enabled = NO;
    
    // Hacemos copia seguridad toolbar para poder eliminar botones ocultos
    
    self.todosElementosToolbar = [self.toolbar.items mutableCopy];
    
    // Eliminamos en una copia temporal los botones a ocultar y se los asignamos al toolbar
    NSMutableArray *items = [self.todosElementosToolbar mutableCopy];
    [items removeObject: self.botonEliminar];
    [items removeObject:self.botonSeleccionarTodo];
    [items removeObject:self.botonTrasladar];
    self.toolbar.items = items;
}

- (void)fetchBiblioteca
{
    //leemos los datos para inicializar la biblioteca
    NSFetchRequest *fetchRequest = [NSFetchRequest fetchRequestWithEntityName:@"CDataColeccion"];
    NSString *cacheName = [@"CDataColeccion" stringByAppendingString:@"Cache"];
    
    //indicamos la columna de ordenacion
    NSSortDescriptor *sortDescriptor = [NSSortDescriptor sortDescriptorWithKey:@"ordenVisualizacion" ascending:YES];
    [fetchRequest setSortDescriptors:@[sortDescriptor]];
    
    //predicado para seleccion de los comics de una coleccion
    NSPredicate *pred = [NSPredicate predicateWithFormat:@"coleccion = %@", self.coleccionActiva];
    [fetchRequest setPredicate:pred];
    
    //almacenamos los datos leidos
    self.fetchedResultsController = [[NSFetchedResultsController alloc] initWithFetchRequest:fetchRequest managedObjectContext:self.managedObjectContext sectionNameKeyPath:nil cacheName:cacheName];

    NSError *error;
    if (![self.fetchedResultsController performFetch:&error])
    {
        NSLog(@"Fallo en Fetch: %@", error);
    }
}

- (void)personalizaEstilo
{
    //inicializamos objeto para carga de preferencias y establecemos los datos.
    PreferenciasUsuario *preferencias = [[PreferenciasUsuario alloc] init];
    
    self.view.backgroundColor = preferencias.backgroundColor;
}


#pragma mark - UICollectionView Datasource
- (NSInteger)collectionView:(UICollectionView *)view numberOfItemsInSection:(NSInteger)section;
{
    //numero de comics a visualizar en collectionview
    return [self.biblioteca getComicsCount];
}

- (NSInteger)numberOfSectionsInCollectionView:(UICollectionView *)collectionView
{
    //numero de secciones del collectionview
    return 1;
}

- (UICollectionViewCell *)collectionView:(UICollectionView *)cv cellForItemAtIndexPath:(NSIndexPath *)indexPath
{
    //generamos la celula que debe visualizarse en el collectionview
    CelulaComic *cell = [cv dequeueReusableCellWithReuseIdentifier:@"CelulaComic" forIndexPath:indexPath];
    cell.backgroundColor = [UIColor whiteColor];

    Comic *comic = [self.biblioteca getComicAtIndex:indexPath.row];
    cell.thumnail = [comic imagenMiniatura];
    
    return cell;
}


#pragma mark - UICollectionViewDelegate

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath
{
    //seleccion de elementos en collectionview
    if (!self.modoEdicion) {
        //si no estamos en modo seleccion abrimos el comic
        Comic *comic = [self.biblioteca getComicAtIndex:indexPath.row];
        [self performSegueWithIdentifier:@"VisualizadorComic" sender:comic];
        [self.collectionView deselectItemAtIndexPath:indexPath animated:YES];
    }
    else
    {
        //si estamos en modo edicion lo añadimos a la lista de comics a editar
        Comic *comic = [self.biblioteca getComicAtIndex:indexPath.row];
        [self.comicsSeleccionados addObject:comic];
        self.botonEliminar.enabled = YES;
        self.botonTrasladar.enabled = YES;
    }
}

- (void)collectionView:(UICollectionView *)collectionView didDeselectItemAtIndexPath:(NSIndexPath *)indexPath
{
    if (self.modoEdicion) {
        //deseleccionamos el comic marcado
        Comic *comic = [self.biblioteca getComicAtIndex:indexPath.row];
        [self.comicsSeleccionados removeObject:comic];
        if ([self.comicsSeleccionados count]==0)
        {
            self.botonTrasladar.enabled = NO;
            self.botonEliminar.enabled = NO;
        }
    }
}

- (CGSize)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout*)collectionViewLayout sizeForItemAtIndexPath:(NSIndexPath *)indexPath
{
    //establecemos el tamaño de la celula del comic a visualizar por parte del collectionview
    CGSize retval = CGSizeMake(150, 200);
    
    //añadimos espacio para el borde
    retval.height += 20;
    retval.width += 20;
    return retval;
}

- (UIEdgeInsets)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout*)collectionViewLayout insetForSectionAtIndex:(NSInteger)section
{
    //separacion entre secciones del collectionview
    return UIEdgeInsetsMake(20, 20, 20, 20);
}

#pragma mark - segue
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    //control de segues para llamar a otros MVCs
    if ([segue.identifier isEqualToString:@"VisualizadorComic"])
    {
        //abrimos la visualizacion de un comic
        VisualizadorComicViewController *vc = segue.destinationViewController;

        vc.comic = sender;
    }
    
    if ([segue.identifier isEqualToString:@"popoverColeccion"] || [segue.identifier isEqualToString:@"popoverTrasladar"])
    {
        //visualizamos el popover coleccion
        if (![self.coleccionPopoverController isPopoverVisible])
        {
            //si la operacion es de traslado lo indicamos para una vez seleccionada la coleccion saber que debemos llevar a cabo el traslado
            if ([segue.identifier isEqualToString:@"popoverTrasladar"])
            {
                self.trasladandoComic = YES;
            }
            [[segue destinationViewController] setDelegate:self];
            [[segue destinationViewController] setManagedObjectContext:self.managedObjectContext];
            
            self.coleccionPopoverController = [(UIStoryboardPopoverSegue *)segue popoverController];
            
            self.coleccionPopoverController.delegate = self;
            ColeccionViewController *popoverFirstTVC = (ColeccionViewController *)((UIPopoverController *) segue.destinationViewController);
            popoverFirstTVC.coleccionActual = self.coleccionActiva;
            
            //pasamos al popover la coleccionactiva en este momento
            popoverFirstTVC.coleccionActual = self.coleccionActiva;
        }
    }
}

- (BOOL)shouldPerformSegueWithIdentifier:(NSString *)identifier sender:(id)sender {
    
    //si ya esta visualizandose el popover no lo volvemos a abrir
    if ([identifier isEqualToString:@"popoverColeccion"])
    {
        if ([self.coleccionPopoverController isPopoverVisible])
            return NO;
        else
            return YES;
    }
    
    return YES;
}

- (void) popoverControllerDidDismissPopover:(UIPopoverController *)popoverController
{
    //pulsacion fuera del popover por lo que debemos actualizar la biblioteca si es necesario
    NSLog(@"popover cerrado");
    
    [self actualizaBiblioteca];    
}

#pragma mark - LXReorderableCollectionViewDelegateFlowLayout methods

- (void)collectionView:(UICollectionView *)theCollectionView layout:(UICollectionViewLayout *)theLayout itemAtIndexPath:(NSIndexPath *)theFromIndexPath willMoveToIndexPath:(NSIndexPath *)theToIndexPath
{
    //arrastamos y soltamos comic a una nueva posicion
    [self.biblioteca moveComicFromIndex:theFromIndexPath.row toIndex:theToIndexPath.row];
}

#pragma mark - ColeccionViewDelegate
-(void) coleccionSeleccionada:(NSString *)nuevaColeccion
{
    NSLog(@"Coleccion Seleccionada: %@", nuevaColeccion);

    //delegado que recibe la coleccion seleccionada en el popover
    
    //acualizamos la interfaz y la coleccionactiva
    [self actualizaBotonLabelColeccion:nuevaColeccion];
    self.coleccionActiva = nuevaColeccion;
    
    //Grabamos como activa para el reinicio de la aplicacion la coleccion seleccionada
    NSUserDefaults * standardUserDefaults = [NSUserDefaults standardUserDefaults];
    
    // saving an NSString
    [standardUserDefaults setObject:nuevaColeccion forKey:@"coleccionActiva"];
    [standardUserDefaults synchronize];
    
    //si habia que trasladar los comics hacemos el traslado a la coleccion seleccionada
    if (self.trasladandoComic) {
        NSLog(@"Trasladando Comic parte 2");
        self.trasladandoComic = NO;
        
        //Trasladamos los comics seleccionados
        for (Comic *comic in [self.comicsSeleccionados copy]) {
            [self.biblioteca trasladaComic:comic toColeccion:nuevaColeccion];
            [self.comicsSeleccionados removeObject:comic];
        }
        //recargamos los datos
        dispatch_async(dispatch_get_main_queue(), ^{
            //recargar datos collectionview
            [self.collectionView reloadData];
        });
    }
    
    //actualizamos la biblioteca
    [self actualizaBiblioteca];
    if ([self.coleccionPopoverController isPopoverVisible])
    {
        //ocultamos popover
        [self.coleccionPopoverController dismissPopoverAnimated:YES];
        self.coleccionPopoverController = nil;
    }
}

- (void)creaColeccionDefecto
{
    //crea la coleccion por defecto "Comics" en core data, la asignamos como activa y ordenamos la coleccion
    CDataColeccion * cdColeccion = (CDataColeccion *) [NSEntityDescription insertNewObjectForEntityForName:@"CDataColeccion" inManagedObjectContext:self.managedObjectContext];
    self.coleccionActiva = @"Comics";
    cdColeccion.coleccion = self.coleccionActiva;
    cdColeccion.ordenVisualizacion = [NSNumber numberWithLong:0];
    
    //commit core data
    NSError *error;
    if (![self.managedObjectContext save:&error])
    {
        NSLog(@"Error en Commit: %@, %@", error, [error userInfo]);
        exit(-1);
    }
    NSUserDefaults * standardUserDefaults = [NSUserDefaults standardUserDefaults];
    
    //grabamos la coleccion en preferencias para recuperarla posteriormente
    [standardUserDefaults setObject:self.coleccionActiva forKey:@"coleccionActiva"];
    [standardUserDefaults synchronize];
}

- (void)compruebaExisteColeccion
{
    //Obtenemos la coleccion actual
    NSUserDefaults *standardUserDefaults = [NSUserDefaults standardUserDefaults];
    
    // getting an NSString object
    self.coleccionActiva = [standardUserDefaults stringForKey:@"coleccionActiva"];
    
    if (self.coleccionActiva == nil)
    {
        //intentamos ver si existe la coleccion en core data
        NSFetchRequest * fetchRequest = [[NSFetchRequest alloc] init];
        
        NSEntityDescription *entity = [NSEntityDescription entityForName:@"CDataColeccion" inManagedObjectContext:self.managedObjectContext];
        [fetchRequest setEntity:entity];
        
        //predicado
        NSPredicate *pred = [NSPredicate predicateWithFormat:@"coleccion = %@", @"Comics"];
        [fetchRequest setPredicate:pred];
        
        //leemos los datos
        NSError *error;
        NSArray *foundObjects = [self.managedObjectContext executeFetchRequest:fetchRequest error:&error];
        if (foundObjects == nil)
        {
            FATAL_CORE_DATA_ERROR(error);
            return;
        }

        if ([foundObjects count] == 0)
        {
            NSLog(@"Ninguna Coleccion Activa");
            //Crear Coleccion por defecto: Comics
        
            [self creaColeccionDefecto];
        }
        else
        {
            //ya existia la coleccion
            self.coleccionActiva = @"Comics";
        }
    }
    else
    {
        //Comprobar Existencia Coleccion
        NSFetchRequest * fetchRequest = [[NSFetchRequest alloc] init];
        
        NSEntityDescription *entity = [NSEntityDescription entityForName:@"CDataColeccion" inManagedObjectContext:self.managedObjectContext];
        [fetchRequest setEntity:entity];
        
        NSPredicate *pred = [NSPredicate predicateWithFormat:@"coleccion = %@", self.coleccionActiva];
        [fetchRequest setPredicate:pred];
        
        NSError *error;
        NSArray *foundObjects = [self.managedObjectContext executeFetchRequest:fetchRequest error:&error];
        if (foundObjects == nil)
        {
            FATAL_CORE_DATA_ERROR(error);
            return;
        }
        
        if ([foundObjects count] == 0) {
            self.coleccionActiva = @"Comics";
        }
        
        NSLog(@"Coleccion Activa: %@", self.coleccionActiva);
    }
}

@end
