//
//  ColeccionViewController.m
//  PowerComicReader
//
//  Created by Sebastian Suelves Selles on 21/04/13.
//  Copyright (c) 2013 Sebastian Suelves Selles. All rights reserved.
//

#import <QuartzCore/QuartzCore.h>
#import "ColeccionViewController.h"
#import "CelulaColeccionCell.h"
#import "CDataColeccion.h"
#import "CDataComic.h"
#import <CoreData/CoreData.h>

@interface ColeccionViewController ()
//IBOutlets
@property (strong, nonatomic) IBOutlet UITableView *tableView;
@property (weak, nonatomic) IBOutlet UIButton *botonNueva;
@property (weak, nonatomic) IBOutlet UIButton *botonEditar;

//control de modo insercion
@property (nonatomic) BOOL insertando;

//Lista de comics por borrar
@property (nonatomic, strong) NSMutableArray *listaColeccionesBorradasSinCommit;
@property (nonatomic, strong) NSIndexPath *indexpathNuevaColeccion;
@end

@implementation ColeccionViewController

//getter
-(NSMutableArray *) listaColeccionesBorradasSinCommit
{
    if (!_listaColeccionesBorradasSinCommit)
    {
        _listaColeccionesBorradasSinCommit = [[NSMutableArray alloc] init];
    }
    return _listaColeccionesBorradasSinCommit;
}

//getter
-(NSMutableArray *) listaColecciones
{
    if (!_listaColecciones)
    {
        _listaColecciones = [[NSMutableArray alloc] init];
    }
    return _listaColecciones;
}

- (IBAction)botonNuevaPulsado:(UIButton *)sender
{
        //CoreDATA

        self.insertando = YES;
        
        CDataColeccion *cdColeccion = (CDataColeccion *)[NSEntityDescription insertNewObjectForEntityForName:@"CDataColeccion" inManagedObjectContext:self.managedObjectContext];
	

        /*
         Para crear una nueva coleccion:
         * Añadimos una nueva fila en el tableview
         * Hacemos Scroll para poner la fila visible
         * Ponemos activo el textview para la edicion del nombre de la coleccion
         */
        //[self.tableView setEditing:YES animated:YES];
        
        NSIndexPath *indexPath = [NSIndexPath indexPathForRow:[self.listaColecciones count] inSection:0];
        self.indexpathNuevaColeccion = indexPath;

        //insercion en el modelo de la nueva coleccion
        [self.listaColecciones insertObject:cdColeccion atIndex:[self.listaColecciones count]];
    
        //insercion en table view de la nueva coleccion
        [self.tableView insertRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];

	
        //llevamos constancia del orden de visualizacion para en la siguiente carga visualizar las coleccione en el mismo orden
        [self updateOrdenVisualizacion];
    
        //establecemos el modo edicion del tableview
        [self setEditing:YES animated:YES];
    
        //nos aseguramos de que sea visible la nueva coleccion para su edicion
        [self.tableView scrollToRowAtIndexPath:indexPath atScrollPosition:UITableViewScrollPositionTop animated:YES];
    
        //obtenemos el textfield de la celula y le mandamos el foco
        CelulaColeccionCell *cell = (CelulaColeccionCell *)[self.tableView cellForRowAtIndexPath:indexPath];
    
        [cell.textField becomeFirstResponder];
    
        //ajustamos la visualizacion del boton
        [self.botonNueva setEnabled:NO];
        self.botonNueva.alpha = 0.65f;
        [self.botonEditar setTitle:@"OK" forState:UIControlStateNormal];
}

- (IBAction)botonEditarPulsado:(UIButton *)sender
{
    //Entramos modo edicion
    if (![self.tableView isEditing] && !self.insertando) //sino estamos insertando ni editando, entramos en modo edicion y cambiamos el boton a OK
    {
        [self.botonEditar setTitle:@"OK" forState:UIControlStateNormal];
        [self.tableView setEditing:YES animated:YES];
    }
    else //pulsamos OK
    {
        //Resestablecemos el boton a editar, y anulamos el modo edicion
        NSString *editar = NSLocalizedString(@"Edit", @"ColeccionViewController Edit");
        [self.botonEditar setTitle:editar forState:UIControlStateNormal];
        [self.tableView setEditing:NO animated:YES];
        
        //Simulamos intro en el teclado
        [self ocultaTeclado];
        
        //actualizamos la vista
        [self.tableView reloadData];
    }
}

- (void)eliminaFilaInsertadasVacias
{
    //eliminamos colecciones sin titulo
    NSUInteger indice=0;
    for (CelulaColeccionCell *cell in self.tableView.visibleCells)
    {
        if ([cell.textField.text isEqualToString:@""])
        {
            NSLog(@"editando");
            [self.listaColecciones removeObjectAtIndex:indice];
        }
        indice++;
    }
}

- (void)ocultaTeclado
{
    //Simula la pulsacion de enter en el teblado
    for (CelulaColeccionCell *cell in self.tableView.visibleCells)
    {
        //buscamos la celula que estamos editando y simulamos el enter
        if (cell.textField.isEditing) {
            NSLog(@"editando");
            [cell.textField resignFirstResponder];
        }
    }
}

- (void)eliminarFicherosColeccionesBorradas
{
    //Borramos todos los elementos pendientes de una tacada
    for (NSInteger indice=0; indice < [self.listaColeccionesBorradasSinCommit count]; indice++)
    {
        //recuperamos el elemento a borrar
        CDataColeccion *elemento =  [self.listaColeccionesBorradasSinCommit objectAtIndex:indice];
        //Borrado de ficheros implicados
        for (CDataComic *comicBorrar in [elemento.coleccionComic allObjects])
        {
            [self borraFicherosComic:comicBorrar];
        }
        //Borramos en Core Data
        [[self managedObjectContext] deleteObject:[self.listaColeccionesBorradasSinCommit objectAtIndex:indice]];
    }
    //limpiamos los comics pendientes
    [self.listaColeccionesBorradasSinCommit removeAllObjects];
}

-(void) borraFicherosComic:(CDataComic *)comic
{
    //Borrado a nivel se sistema de fichero de los ficheros que forman un comic
    NSArray* paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDir = [paths objectAtIndex:0];
    
    NSString *fichBorrar = [documentsDir stringByAppendingPathComponent:@"Imported Comics"];
    fichBorrar = [fichBorrar stringByAppendingPathComponent:comic.nombreCarpeta];
    NSLog(@"Directorio a borrar: %@", fichBorrar);
    [[NSFileManager defaultManager] removeItemAtPath:fichBorrar error:NULL];
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    //configuramos tableview
    self.tableView.layer.cornerRadius = 8.0;
    self.tableView.allowsSelectionDuringEditing = YES;
    
    //inicializamos la lista de colecciones
    self.listaColecciones = [NSMutableArray array];
    
    //leemos la entidad CDDataColeccion
    NSFetchRequest * fetchRequest = [[NSFetchRequest alloc] init];
    
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"CDataColeccion" inManagedObjectContext:self.managedObjectContext];
    [fetchRequest setEntity:entity];
    
    //Fijamos ordenacion por campo ordenVisualizacion
    NSSortDescriptor *sortDescriptor = [NSSortDescriptor sortDescriptorWithKey:@"ordenVisualizacion" ascending:YES];
    [fetchRequest setSortDescriptors:[NSArray arrayWithObject:sortDescriptor]];
    
    //Ejecutamos la consulta
    NSError *error;
    NSArray *foundObjects = [self.managedObjectContext executeFetchRequest:fetchRequest error:&error];
    if (foundObjects == nil)
    {
        FATAL_CORE_DATA_ERROR(error);
        return;
    }
    
    //Asignamos a nuestro modelo los datos leidos
    self.listaColecciones = [foundObjects mutableCopy];
    //Ponemos el tableview en primer plano para que sea este el que reciba los eventos tactiles
    [self.view bringSubviewToFront:self.tableView];
}



#pragma mark - Table view data source
-(NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    //secciones del tableview
    return 1;
}

-(NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    //numeros de colecciones a representar
    return [self.listaColecciones count];
}

-(UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    //montamos la celula a mostrar a partir de los datos del modelo
    static NSString *cellIdentifier = @"ColeccionCell";
    CelulaColeccionCell * cell = [tableView dequeueReusableCellWithIdentifier:cellIdentifier];
    
    NSLog(@"indexpath:%ld - lista:%@", (long)indexPath.row, [self.listaColecciones objectAtIndex:indexPath.row]);
    
    cell.textField.text = [[self.listaColecciones objectAtIndex:indexPath.row] coleccion];
    
    cell.textField.tag = indexPath.row;
    
    //como la coleccion Comics no puede borrarse le metemos un factor alpha para que se muestre como deshabilitada
    if ([cell.textField.text isEqualToString:@"Comics"] || [cell.textField.text isEqualToString:self.coleccionActual])
    {
        cell.textField.alpha = 0.65f;
    }
    
    return cell;
}

- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    //controlamos que no se pueda editar la coleccion por defecto "Comics"
    if ([[[self.listaColecciones objectAtIndex:indexPath.row] coleccion] isEqualToString:@"Comics"]
        || [[[self.listaColecciones objectAtIndex:indexPath.row] coleccion] isEqualToString:self.coleccionActual])
    {
        return NO;
    }
    else
    {
        //cualquier otra coleccion es editable
        return YES;
    }
}

#pragma mark - Table view delegate

- (NSIndexPath *)tableView:(UITableView *)tableView willSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    //control para evitar efectos colaterales insertando
    if (self.insertando)
    {
        if ([self.indexpathNuevaColeccion isEqual:indexPath])
        {
            return indexPath;
        }
        else
        {
            return nil;
        }
    }
    
    return indexPath;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    NSLog(@"didSelect");
    
    //sino estamos editando
    if (!self.tableView.isEditing)
    {
        // Devolvemos la coleccion seleccionada (modo no edicion)

        NSString *nombreColeccion = [[self.listaColecciones objectAtIndex:indexPath.row] coleccion];
    
        if (self.delegate != nil)
        {
            //mandamo a traves del protocolo la coleccionSeleccionada
            [self.delegate coleccionSeleccionada:nombreColeccion];
        }
    }
}

-(void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)sourceIndexPath toIndexPath:(NSIndexPath *)destinationIndexPath
{
    //traladamos una coleccion de sitio en el tableview, para ello movemos en el modelo y actualizamos el orden de visualizacion
    CDataColeccion *coleccion = [self.listaColecciones objectAtIndex:sourceIndexPath.row];
    
    [self.listaColecciones removeObjectAtIndex:sourceIndexPath.row];
    [self.listaColecciones insertObject:coleccion atIndex:destinationIndexPath.row];
    
    [self updateOrdenVisualizacion];
}

//Borrado Elementos Table View
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (editingStyle == UITableViewCellEditingStyleDelete) //borrado de una coleccion
    {
        NSLog(@"row: %ld, count colecciones: %lu, count borra: %lu",(long)indexPath.row, (unsigned long)[self.listaColecciones count], (unsigned long)[self.listaColeccionesBorradasSinCommit count]);
        
        //anotamos que coleccion queremos borrar
        [self.listaColeccionesBorradasSinCommit addObject:[self.listaColecciones objectAtIndex:indexPath.row]];

        NSLog(@"row: %ld, count colecciones: %lu, count borra: %lu",(long)indexPath.row, (unsigned long)[self.listaColecciones count], (unsigned long)[self.listaColeccionesBorradasSinCommit count]);
        

        //Actualizamos TableView y modelo
        [self.listaColecciones removeObjectAtIndex:indexPath.row];
        [tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationAutomatic];
        
        //eliminamos los ficheros de la coleccion
        [self eliminarFicherosColeccionesBorradas];
        
        // Commit.
        NSError *error;
        if (![self.managedObjectContext save:&error])
        {
            // Handle the error.
            NSLog(@"Error en commit: %@, %@", error, [error userInfo]);
            exit(-1);  // Fail
        }

        [self updateOrdenVisualizacion];
    }
}

- (void)setEditing:(BOOL)editing animated:(BOOL)animated
{
    //entrada modo edicion
    [super setEditing:editing animated:animated];
    for (CelulaColeccionCell *cell in [self.tableView visibleCells])
    {
        //habilitamos los textfields que estaban desactivados fuera del modo edicion, para ello recuperamos el textfield de la celula y lo modificamos
        UITextField *tf = cell.textField;
        tf.enabled = editing;
    }
}

- (void) updateOrdenVisualizacion
{
    //actualizamos los tags
    [self updateRowTags];
    //actualizamos en core data el orden de visualizacion
    [self updateRowOrdenVisualizacion];
}

- (void)updateRowTags
{
    //asignamos un tag ascendente a cada celula del tableview en funcion de su fila de visualizacion
	NSArray *visibleCells = [self.tableView visibleCells];
	for (CelulaColeccionCell *cell in visibleCells)
    {
		NSInteger tag = [[self.tableView indexPathForCell:cell] row];
		cell.textField.tag = tag;
	}
}

- (void)updateRowOrdenVisualizacion
{
    //actualizamos el orden de visualizacion
    for (NSInteger indice=0; indice<[self.listaColecciones count]; indice++)
    {
        CDataColeccion *elemento = [self.listaColecciones objectAtIndex:indice];
        [elemento setOrdenVisualizacion:[NSNumber numberWithLong:indice]];
        [self.listaColecciones replaceObjectAtIndex:indice withObject:elemento];
	}
}


#pragma mark -
#pragma mark Editing text fields

- (BOOL)textFieldShouldEndEditing:(UITextField *)textField
{
    //finalizamos la edicion (al esconder el teclado)
    
    //simulamos haber pulsado el boton OK
    [self botonEditarPulsado:nil];
    
    //reestablecemos los valores al salir del modo edicion/insercion para que los botones aparezcan correctamente
    self.insertando = NO;
    self.botonNueva.enabled = YES;
    self.botonNueva.alpha = 1.0f;
    
    
    //Grabamos los cambios (actualizacion/insercion) si el titulo de la coleccion no es nulo
    if (![textField.text isEqualToString:@""])
    {
        CDataColeccion *col = [self.listaColecciones objectAtIndex:textField.tag];
        [col setColeccion:textField.text];
        col.ordenVisualizacion = [NSNumber numberWithLong:textField.tag];
    
        [self.listaColecciones replaceObjectAtIndex:textField.tag withObject:col];

        //actualizamos el orden de visualizacion
        [self updateOrdenVisualizacion];
        
        //Core Data
        // Commit.
        NSError *error;
        if (![self.managedObjectContext save:&error]) {
            // Handle the error.
            NSLog(@"Error en commit: %@, %@", error, [error userInfo]);
            exit(-1);  // Fail
        }
    }
    else
    {
        //Si el titulo es nulo de la coleccion deshacemos los cambios
        [self eliminaFilaInsertadasVacias];
        [self.managedObjectContext rollback];
    }
    
	return YES;
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField
{
    //escondemos el teclado si efectivamente se pulsa enter
	[textField resignFirstResponder];
    
	return YES;
}


@end
