//
//  Comic.m
//  PowerComicReader
//
//  Created by Sebastian Suelves Selles on 10/03/13.
//  Copyright (c) 2013 Sebastian Suelves Selles. All rights reserved.
//

#import "Comic.h"

@interface Comic ()
@property (strong, nonatomic, readwrite) NSNumber *paginaActual;
@property (strong, nonatomic, readwrite) NSString *directorio;
@property (strong, nonatomic, readwrite) NSNumber *numeroPaginas;
@property (strong, nonatomic, readwrite) NSString *extensionImagenes;
@property (strong, nonatomic, readonly) NSString *directorioDocuments;
@property (strong, nonatomic, readonly) NSString *directorioComicsImportados;
@property (strong, nonatomic, readonly) NSString *directorioComic;
@end

@implementation Comic

- (id) init
{
    return [self initWithColeccion:@"Comics" AndOrdenColeccion:@1 AndPaginaActual:@0 AndNumPaginas:@1 AndDirectorio:@"" AndExtensionImagenes:@".jpg"];
}

//inicializador Designado
- (id) initWithColeccion:(NSString *)coleccion AndOrdenColeccion:(NSNumber *)ordenColeccion AndPaginaActual:(NSNumber *)paginaActual AndNumPaginas:(NSNumber *)numeroPaginas AndDirectorio:(NSString *)directorio AndExtensionImagenes:(NSString *)extensionImagenes
{
    self = [super init];
    
    if (self)
    {
        //inicializmos el objeto
        _coleccion = coleccion;
        _ordenColeccion = ordenColeccion;
        _paginaActual = paginaActual;
        _numeroPaginas = numeroPaginas;
        _directorio = directorio;
        _extensionImagenes = extensionImagenes;
    }
    return self;
}

- (NSString *) directorioDocuments
{
    //Calculamos el directorio Documents de la aplicacion
    NSString *documentsDir;
    
    NSArray* paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    documentsDir = [paths objectAtIndex:0];
    
    return documentsDir;
}

- (NSString *) directorioComicsImportados
{
    //Calculamos el directorio de comics importados
    NSString *importedDir;
    
    importedDir = [NSString stringWithFormat:@"%@/Imported Comics/", self.directorioDocuments];
    
    return importedDir;
}

- (NSString *) directorioComic
{
    //calculamos el directorio del comic
    NSString *comicDir;
    
    comicDir = [NSString stringWithFormat:@"%@%@/", self.directorioComicsImportados, self.directorio];
    
    return comicDir;
}

- (UIImage *)paginaAnterior
{
    //devolvemos la imagen de la pagina anterior
    NSString *ficheroImagen;
    
    if (![self.paginaActual isEqualToNumber:@1])
    {
        unsigned int pagina = [self.paginaActual unsignedIntValue] - 1;
        
        self.paginaActual = [NSNumber numberWithUnsignedInt:pagina];
    }
    
    ficheroImagen = [NSString stringWithFormat:@"%@%@%@", self.directorioComic, [self.paginaActual stringValue], self.extensionImagenes];

    return [UIImage imageWithContentsOfFile:ficheroImagen];
}

- (UIImage *)paginaSiguiente
{
    //devolvemos la imagen de la pagina siguiente
    NSString *ficheroImagen;
    
    if (![self.paginaActual isEqualToNumber:self.numeroPaginas])
    {
        unsigned int pagina = [self.paginaActual unsignedIntValue] + 1;
        
        self.paginaActual = [NSNumber numberWithUnsignedInt:pagina];
    }
    
    ficheroImagen = [NSString stringWithFormat:@"%@%@%@", self.directorioComic, [self.paginaActual stringValue], self.extensionImagenes];

    return [UIImage imageWithContentsOfFile:ficheroImagen];
}

- (UIImage *)imagenActual
{
    //devolvemos la pagina actual
    NSString *ficheroImagen;
    
    ficheroImagen = [NSString stringWithFormat:@"%@%@%@", self.directorioComic, [self.paginaActual stringValue], self.extensionImagenes];

    return [UIImage imageWithContentsOfFile:ficheroImagen];
}

- (UIImage *)imagenMiniatura
{
    //devolvemos la imagen de la miniatura
    NSString *ficheroMiniatura;
    
    ficheroMiniatura = [NSString stringWithFormat:@"%@0.png", self.directorioComic];

    return [UIImage imageWithContentsOfFile:ficheroMiniatura];
}

@end
