//
//  VisualizadorComicViewController.m
//  PowerComicReader
//
//  Created by Sebastian Suelves Selles on 26/03/13.
//  Copyright (c) 2013 Sebastian Suelves Selles. All rights reserved.
//

#import "VisualizadorComicViewController.h"
#import "Biblioteca.h"
#import "PreferenciasUsuario.h"




#define ZOOM_VIEW_TAG 100

@interface VisualizadorComicViewController () <UIScrollViewDelegate>
//IBOutlets
@property (strong, nonatomic) IBOutlet UIScrollView *scrollView;
@property (strong, nonatomic) IBOutlet UIImageView *imageView;
@property (strong, nonatomic) IBOutlet UIToolbar *toolbar;

//propiedades para el almacenamiento de las preferencias
@property (nonatomic) BOOL usarDobleTapParaZoomInOut;
@property (strong, nonatomic) NSString *nivelZoomDobleTap;
@property (nonatomic)BOOL modoZoomIn;
@end

@implementation VisualizadorComicViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
	
    //cargamos las preferencias relativas al zoom
    [self preferenciasZoom];
    
    self.toolbar.hidden = YES;
    
    //establecemos los niveles maximo y minimo de zoom
    self.scrollView.minimumZoomScale = 1.0;
    self.scrollView.maximumZoomScale = 4.0;
    
    self.scrollView.delegate = self;
    
    [self.imageView setTag:ZOOM_VIEW_TAG];
    
    // añadimos reconocedores de gestos al image view
    
    //configuramos el reconocimiento de un tap simple
    UITapGestureRecognizer *singleTap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(handleSingleTap:)];
    
    //si esta habilitado el uso de doble tap añadimos un reconocedor para este
    if (self.usarDobleTapParaZoomInOut)
    {
        UITapGestureRecognizer *doubleTap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(handleDoubleTap:)];
        [doubleTap setNumberOfTapsRequired:2];
        [self.imageView addGestureRecognizer:doubleTap];
        
        //para evitar interferencias entre el tap simple y doble habilitamos que para que se de un tap simple no deba haber tenido exito uno doble
        [singleTap requireGestureRecognizerToFail:doubleTap];
    }
    
    //habilitamos el reconocimiento de gestos de desplazamiento de laterales
    UISwipeGestureRecognizer *swipeGestureRight = [[UISwipeGestureRecognizer alloc] initWithTarget:self action:@selector(handleSwipeRight:)];
    [swipeGestureRight setDirection:UISwipeGestureRecognizerDirectionRight];
    
    UISwipeGestureRecognizer *swipeGestureLeft = [[UISwipeGestureRecognizer alloc] initWithTarget:self action:@selector(handleSwipeLeft:)];
    [swipeGestureLeft setDirection:UISwipeGestureRecognizerDirectionLeft];
    
    //Añadimos los gestos configurados al imageview
    [self.imageView addGestureRecognizer:singleTap];
    
    [self.imageView addGestureRecognizer:swipeGestureRight];
    [self.imageView addGestureRecognizer:swipeGestureLeft];
    
    
    //establecemos como escala de zoom 1.0 (sin zoom)
    self.scrollView.zoomScale = 1.0;
    
    //mostramos la primera imagen actual del comic
    UIImage *image = [self.comic imagenActual];
    self.imageView.image = image;
    
    // calculate minimum scale to perfectly fit image width, and begin at that scale
    float minimumScale = [self.scrollView frame].size.width  / [self.imageView frame].size.width;
    [self.scrollView setMinimumZoomScale:minimumScale];
    [self.scrollView setZoomScale:minimumScale];

}

- (void)preferenciasZoom
{
    //creamos un objeto de preferencias de usuario y guardamos en las propiedades del MVC los valores obtenidos
    PreferenciasUsuario *preferencias = [[PreferenciasUsuario alloc] init];
    self.usarDobleTapParaZoomInOut = preferencias.usarZoomDobleTap;
    self.nivelZoomDobleTap = preferencias.nivelZoomDobleTap;
}

- (IBAction)botonBiblioteca:(UIBarButtonItem *)sender {
    [self.presentingViewController dismissViewControllerAnimated:YES completion:^{}];
}

- (void)showToolbar:(BOOL)show
{
    //ocultamos o mostramos la toolbar para volver al MVC BibliotecaViewController
    if (show == YES && self.toolbar.hidden == YES) {
        
        // Cambiamos las coordenadas para que esten fuera de la Vista y quede oculta
        CGRect frame = self.toolbar.frame;
        frame.origin.y = -frame.size.height;
        self.toolbar.frame = frame;
        
        //anima la desaparicion del toolbar
        self.toolbar.hidden = NO;
        frame.origin.y = 0.0;
        [self.view bringSubviewToFront:self.toolbar];
        
        [UIView animateWithDuration:0.3
                         animations:^(void) {
                             self.toolbar.frame = frame;
                         }
         ];
    }
    else if (show == NO && self.toolbar.hidden == NO) {
        
        //reestablece las coordenadas del toolbar para que sea visible
        CGRect frame = self.toolbar.frame;
        
        //anima la aparicion del toolbar
        frame.origin.y = -frame.size.height;
        [self.view bringSubviewToFront:self.toolbar];
        
        [UIView animateWithDuration:0.3
                         animations:^(void) {
                             self.toolbar.frame = frame;
                         }
                         completion:^(BOOL finished) {
                             self.toolbar.hidden = YES;
                         }
         ];
    }
}

#pragma mark UIScrollViewDelegate methods

- (UIView *)viewForZoomingInScrollView:(UIScrollView *)scrollView {
    //zoom para pitch
    return [self.scrollView viewWithTag:ZOOM_VIEW_TAG];
}

//Tomado de los ejemplos de Apple
/************************************** NOTE **************************************/
/* The following delegate method works around a known bug in zoomToRect:animated: */
/* In the next release after 3.0 this workaround will no longer be necessary      */
/**********************************************************************************/
- (void)scrollViewDidEndZooming:(UIScrollView *)scrollView withView:(UIView *)view atScale:(float)scale {
    [scrollView setZoomScale:scale+0.01 animated:NO];
    [scrollView setZoomScale:scale animated:NO];
}

#pragma mark TapDetectingImageViewDelegate methods

- (void)handleSingleTap:(UIGestureRecognizer *)gestureRecognizer {
    // gestor de tap simple
    
    CGPoint position = [gestureRecognizer locationInView:gestureRecognizer.view];
    NSLog(@"posicion (%f, %f)", position.x * self.scrollView.zoomScale, position.y * self.scrollView.zoomScale);
    NSLog(@"scala: %f - %f", self.imageView.image.scale, self.scrollView.zoomScale);
    
    // si el tap se produce en el borde izquierdo vamos a la pagina anterior (sino estamos en modo zoom)
    if ((position.x<200) && (self.scrollView.zoomScale == 1.0)) //pulsacion en el lado izquierdo
    {
        //Mostrar pagina Anterior
        self.imageView.image = [self.comic paginaAnterior];
        [self resetZoom];
    }
    else if ((position.x>self.imageView.frame.size.width-200) && (self.scrollView.zoomScale == 1.0)) //pulsacion en el lado derecho
    {
        //Mostrar pagina siguiente
        self.imageView.image = [self.comic paginaSiguiente];
        [self resetZoom];
    }
    else //pulsacion en el centro, conmutar toolbar
    {
        BOOL toolbarVisible = !self.toolbar.hidden;
        toolbarVisible = !toolbarVisible;
        [self showToolbar:toolbarVisible];
    }
}

- (void)handleDoubleTap:(UIGestureRecognizer *)gestureRecognizer {
    // double tap zooms in
    float newScale;
    
    //invertimos entre modo zoom in/out segun el estado anterior
    self.modoZoomIn = !self.modoZoomIn;
    
    if (self.modoZoomIn) //hacemos zoom in el factor indicado en las preferencias
    {
        newScale = [self.scrollView zoomScale] * [self.nivelZoomDobleTap floatValue];
    }
    else
    {
        newScale = 1.0; //zoom out, restauramos la escala a 1.0
    }
    
    //actualizamos la vista
    CGRect zoomRect = [self zoomRectForScale:newScale withCenter:[gestureRecognizer locationInView:gestureRecognizer.view]];
    [self.scrollView zoomToRect:zoomRect animated:YES];
}

- (void) handleSwipeRight:(UISwipeGestureRecognizer *)gestureRecognizer {
    //gesto deslizamiento a la derecha, ir pagina anterior
    if (self.scrollView.zoomScale == 1.0)
    {
        if (gestureRecognizer.direction & UISwipeGestureRecognizerDirectionRight) {
            //Mostrar pagina Anterior
            self.imageView.image = [self.comic paginaAnterior];
            [self resetZoom];
        }
    }
}

- (void) handleSwipeLeft:(UISwipeGestureRecognizer *)gestureRecognizer {
    //gesto deslizamiento a la izquierda, ir pagina siguiente
    if (self.scrollView.zoomScale == 1.0)
    {
        if (gestureRecognizer.direction & UISwipeGestureRecognizerDirectionLeft) {
            //Mostrar pagina siguiente
            self.imageView.image = [self.comic paginaSiguiente];
            [self resetZoom];
        }
    }
}
#pragma mark Utility methods

- (CGRect)zoomRectForScale:(float)scale withCenter:(CGPoint)center {
    
    CGRect zoomRect;
    
    //ajuste del marco de la imagen respecto a la escala
    zoomRect.size.height = [self.scrollView frame].size.height / scale;
    zoomRect.size.width  = [self.scrollView frame].size.width  / scale;
    zoomRect.origin.x    = center.x - (zoomRect.size.width  / 2.0);
    zoomRect.origin.y    = center.y - (zoomRect.size.height / 2.0);
    
    return zoomRect;
}

- (void)resetZoom
{
    //reseteamos el nivel de zoom
    if (self.scrollView.zoomScale != 1.0)
    {
        [self.scrollView setZoomScale:1.0];
    }
}
@end
